;; intro state
;;
;; Tell player about goal and rules.
;;
;; - display background
;; - display bulldozer, side view
;; - display large rock in front of bulldozer
;; - play sound of engine starting
;; - transition to play state just before the end of the sound clip
;;

(local sound (require "sound"))

(local rock-cut (love.graphics.newImage "assets/background.png"))
(local bdozer-side (love.graphics.newImage "assets/bulldozer-side.png"))
(local big-rock-side (love.graphics.newImage "assets/big-rock-side.png"))
;(local intro-font (love.graphics.newFont "assets/FSEX300.ttf" 32))

(var message {1 "Drive the bulldozer. Push the big rocks."
              2 "Press space to skip the rest of the introduction."
              3 "Use Control Q to quit the game."})

(var message-width {})

(var line-height 50)
(var offset-y (- (/ (* (length message) line-height) 2)))


; count down for intro, can be interrupted
; the starting engine sound clip is 26.05 seconds long
;
(var counter 26)  ; in seconds


(fn activate [game-state]
  (sound.play :starting)
  ; calculate the width of each line in the message
  (love.graphics.setFont game-state.game-font)
  (for [i 1 (# message)]
    (tset message-width i (game-state.game-font:getWidth (. message i)))))

(fn draw [game-state]
; (love.graphics.setFont intro-font)
  (love.graphics.draw rock-cut)
  (love.graphics.draw bdozer-side 300 150)
  ; need to calculate space title needs and choose x from middle
  ; (- (/ w 2) title-width)
  (love.graphics.setFont game-state.title-font)
  (love.graphics.print "B u l l d o z e r" 150 600)
  (love.graphics.setFont game-state.game-font)
  ;; let this code be re-used by pause/help mode
  (for [i 1 (# message)]
    (love.graphics.print (. message i) 100 (+ (* 30 i) 675))))

(fn update [dt game-state set-state]
  (set counter (- counter dt))
  (when (<= counter 0)
    (set-state :play)))

(fn keypressed [key game-state set-state]
  (when (= key "space")
    (set-state :play)))

{:activate activate
 :draw draw
 :update update
 :keypressed keypressed}

