local sti = require("lib.sti")
local sound = require("sound")
local epsilon = 1e-06
local close_dir = (math.pi / 16)
local function normalize_rad(a)
  return math.fmod(a, (2 * math.pi))
end
local function normalize_body_angular(b)
  local angle = b:getAngle()
  return b:setAngle(normalize_rad(angle))
end
local function vector_sum(x_1, y_1, x_2, y_2)
  return (x_1 + x_2), (y_1 + y_2)
end
local function sqr(x)
  return (x * x)
end
local function sum_of_squares(a, b)
  return (sqr(a) + sqr(b))
end
local pixels_per_meter = 20
love.physics.setMeter(pixels_per_meter)
local function pixels_to_meters(n)
  return (n / pixels_per_meter)
end
local function meters_to_pixels(n)
  return (n * pixels_per_meter)
end
local map = sti("assets/map-test.lua", {"box2d"})
local world = love.physics.newWorld(0.0, 0.0)
local low_gear = 1
local high_gear = 2
local dozer = {["accel-force"] = 200000, ["brake-force"] = 700000, ["brake-turn-force"] = 4000, ["max-speed"] = 25, ["max-speed-2"] = 625, ["max-turn-rate"] = (math.pi / 8), ["origin-x"] = 33, ["origin-y"] = 21, ["scale-x"] = 1.0, ["scale-y"] = 1.0, ["turn-force"] = 2000, dir = math.rad(0), gear = low_gear, height = 41, img = love.graphics.newImage("assets/bulldozer-player.png"), width = 65, x = 400, y = 300}
local game_state
local function echo(s, msg)
  return table.insert(s.messages, 1, msg)
end
game_state = {dozer = dozer, echo = echo, flags = {}, map = map, world = world}
gs = game_state
local function move_bulldozer(dt)
  if love.keyboard.isDown("right", "d") then
    if not love.keyboard.isDown("left", "a") then
      local turn_rate = (dozer.body):getAngularVelocity()
      if ((turn_rate - epsilon) <= dozer["max-turn-rate"]) then
        do end (dozer.body):setAngularVelocity(dozer["max-turn-rate"])
        do end (dozer.body):applyTorque(0.0)
      else
        do end (dozer.body):applyTorque(dozer["turn-force"])
      end
    end
  elseif love.keyboard.isDown("left", "a") then
    if not love.keyboard.isDown("right", "d") then
      local turn_rate = (dozer.body):getAngularVelocity()
      if ((turn_rate + epsilon) >= ( - dozer["max-turn-rate"])) then
        do end (dozer.body):setAngularVelocity(( - dozer["max-turn-rate"]))
        do end (dozer.body):applyTorque(0.0)
      else
        do end (dozer.body):applyTorque(( - dozer["turn-force"]))
      end
    end
  else
    local turn_rate = (dozer.body):getAngularVelocity()
    if (math.abs(turn_rate) < epsilon) then
      do end (dozer.body):setAngularVelocity(0.0)
      do end (dozer.body):applyTorque(0.0)
    elseif (turn_rate < 0.0) then
      do end (dozer.body):applyTorque(dozer["brake-turn-force"])
    else
      do end (dozer.body):applyTorque(( - dozer["brake-turn-force"]))
    end
  end
  local point_dir = (dozer.body):getAngle()
  local speed_x, speed_y = (dozer.body):getLinearVelocity()
  local move_dir = math.atan2(speed_y, speed_x)
  local max_speed = (dozer["max-speed"] * dozer.gear)
  if love.keyboard.isDown("up", "w") then
    if not love.keyboard.isDown("down", "s") then
      local max_speed_x = (math.cos(point_dir) * max_speed)
      local max_speed_y = (math.sin(point_dir) * max_speed)
      if (((sum_of_squares(speed_x, speed_y) + epsilon) < dozer["max-speed-2"]) and (math.abs(normalize_rad((point_dir - move_dir))) < close_dir)) then
        local force_x = (math.cos(point_dir) * (dozer["accel-force"] / dozer.gear))
        local force_y = (math.sin(point_dir) * (dozer["accel-force"] / dozer.gear))
        return (dozer.body):applyForce(force_x, force_y)
      else
        do end (dozer.body):setLinearVelocity(max_speed_x, max_speed_y)
        return (dozer.body):applyForce(0.0, 0.0)
      end
    end
  elseif love.keyboard.isDown("down", "s") then
    if not love.keyboard.isDown("up", "w") then
      local max_speed_x = (math.cos(point_dir) * ( - max_speed))
      local max_speed_y = (math.sin(point_dir) * ( - max_speed))
      if (((sum_of_squares(speed_x, speed_y) + epsilon) < dozer["max-speed-2"]) and (math.abs(normalize_rad((point_dir - move_dir))) > close_dir)) then
        local force_x = ( - (math.cos(point_dir) * (dozer["accel-force"] / dozer.gear)))
        local force_y = ( - (math.sin(point_dir) * (dozer["accel-force"] / dozer.gear)))
        return (dozer.body):applyForce(force_x, force_y)
      else
        do end (dozer.body):setLinearVelocity(max_speed_x, max_speed_y)
        return (dozer.body):applyForce(0.0, 0.0)
      end
    end
  else
    if ((math.abs(speed_x) < epsilon) and (math.abs(speed_y) < epsilon)) then
      do end (dozer.body):setLinearVelocity(0.0, 0.0)
      return (dozer.body):applyForce(0.0, 0.0)
    else
      local force_x = ( - (math.cos(move_dir) * (dozer["brake-force"] / dozer.gear)))
      local force_y = ( - (math.sin(move_dir) * (dozer["brake-force"] / dozer.gear)))
      return (dozer.body):applyForce(force_x, force_y)
    end
  end
end
local function activate()
  sound.play("running")
  map:box2d_init(world)
  local player_layer = map:addCustomLayer("PlayerLayer", 3)
  game_state["player-layer"] = player_layer
  player_layer.collidable = true
  player_layer.sprites = {dozer = {image = dozer.img, r = dozer.dir, x = dozer.x, y = dozer.y}}
  dozer.body = love.physics.newBody(world, dozer.x, dozer.y, "dynamic")
  dozer.shape = love.physics.newRectangleShape(pixels_to_meters(dozer.width), pixels_to_meters(dozer.height))
  dozer.fixture = love.physics.newFixture(dozer.body, dozer.shape, 1)
  do end (dozer.body):setInertia(3000)
  do end (dozer.body):setMass(3000)
  player_layer.draw = function()
    local sprite = player_layer.sprites.dozer
    local x, y = (dozer.body):getPosition()
    local dir = (dozer.body):getAngle()
    sprite.x = x
    sprite.y = y
    sprite.r = dir
    return love.graphics.draw(dozer.img, x, y, dir, dozer["scale-x"], dozer["scale-y"], dozer["origin-x"], dozer["origin-y"])
  end
  player_layer.update = function(dt)
    return move_bulldozer(dt)
  end
  return player_layer.update
end
local function draw()
  map:draw()
  return map:box2d_draw(0, 0, 1, 1)
end
local function update(dt)
  world:update(dt)
  map:update(dt)
  return normalize_body_angular(dozer.body)
end
local function keypressed(key, set_state)
  if (("lshift" == key) or ("rshift" == key)) then
    if (dozer.gear == low_gear) then
      dozer.gear = high_gear
      return nil
    else
      dozer.gear = low_gear
      return nil
    end
  end
end
return {activate = activate, draw = draw, keypressed = keypressed, update = update}
