;; simple bulldozer game
;;
;; will build it in stages
;; started with EXE encounter 667 by Phil Hagelberg
;;

;; This module contains non-game-specific bits and mode-changing logic.
;;
(local repl (require "lib.stdio"))

(local sound (require "sound"))

; How do I share these across states?
;
(var game-state {:title-font (love.graphics.newFont 32)
                 :game-font (love.graphics.newFont 20)
                 :width 1000
                 :height 800
                 :scale 1
                 :version "0.6" })

(local canvas (love.graphics.newCanvas game-state.width game-state.height))

(var mode {})  ; start with an empty table

; used to move from one mode to the other
; each mode defines functions and puts them in a table
; the wrap module executes these functions when needed
;
(fn set-state [mode-name ...]
  (set mode (require mode-name))
  (when mode.activate
    (mode.activate ...)))

; start with the introduction
; use the set-state function to force execution of the activation function
; (mode was previously set manually)
;
(set-state :intro game-state)

(fn love.load []
  (: canvas :setFilter "nearest" "nearest")
  (repl.start))

(fn love.draw []
  (love.graphics.setCanvas canvas)
  (love.graphics.clear)
  (love.graphics.setColor 1 1 1)
  (mode.draw game-state)
  (love.graphics.setCanvas)
  (love.graphics.setColor 1 1 1)
  (love.graphics.draw canvas 0 0 0 game-state.scale game-state.scale))

(fn love.update [dt]
  (when mode.update
    (mode.update dt game-state set-state)))

(fn love.quit []
  )

(fn love.keypressed [key]
  (if
    (and (love.keyboard.isDown "lctrl" "rctrl")
           (or (= key "q") (= key "x")))
      (love.event.quit)

    (love.keyboard.isDown "m")
      (sound.toggle)

    :else
      (mode.keypressed key game-state set-state)))

